/////////////////////////////////////////////////////////////////////////////
// concor                                                                    
//               Analyzing the contributions of each subset of variables Yj
//  
//                         to its link with another set X.                       
//                                                                         
//                                                   SUCCESSIVE SOLUTIONS     
//////////////////////////////////////////////////////////////////////////////

function [u,v,V,cov2]=concor(X,Y,py,r)

//////////////////////////////////////////////////////////////////////////////
// X and Y are 2 data matrices, n x p and n x q, of p variables and q variables
// (centered) measured on the same set of n cases.
// The row vector py contains the ky numbers of variables of each of the 
// ky subsets of Y. sum(py)=q.
// Y is the concatenated matrix of ky matrices Yj, j=1,...,ky.
//              
// r is the wanted number of solutions [0<r< 1+min(rank(Yj'X))], and thus  
// rmax < 1+min(min(py),n).
// 
// u (p x r) are the orthonormed axes of X. They are associated to :
// V (q x r) the orthonormed global axes of Y, and to :
// the sub-blocks vj of v (q x r), the orthonormed partial axes of Yj.
// The ky x r matrix cri contains the values cov2(Yj vj(:,k), Xu(:,k)), k=1...r,
// which  for each k are the measures of the relative links of Yj with X.
//
// For each solution k, sumj cov2(Yjvj(:,k),Xu(:,k)) is equal to
// cov2(Xu(:,k),YV(:,k)), and
// that corresponds to these 2  independently optimized criteria, with respective
// norm constraints on the axes vj(:,k) and u(:,k), or on the axes u(:,k) and
// V(:,k);
// Each column of YV represents a mean component of the respective 
// partial components Yjvj.
// For a set of r solutions, the matrix u'X'YV is diagonal and the matrices 
// u'X'Yjvj are triangular.
//
// Example of use : 
// To make some "GPA" : so, by posing the compromise X = Y, "procrustes" rotations
// to the compromise X then are : Yj*(vj*u').
// 
//  
// Lafosse & Hanafi (1997) in Revue de Statistique Appliquee.
// (concor.m corresponds to the svdcp.m function)  
// hanafi@enitiaa-nantes.fr
// Roger.Lafosse@lsp.ups-tlse.fr
/////////////////////////////////////////////////////////////////////////////////
[n,p]=size(X);
q=size(Y,2);
 if (sum(py) ~= q)
    display(' py IS NOT SUITABLE')
    return
 else
 end

ky=size(py,2);
cri=zeros(ky,r);
cumuly=cumsum([0,py]);
u=zeros(p,r);
V=zeros(q,r);
v=V;

for i=1:r
  [p,d,q]=svd(X'*Y);
  u(:,i)=p(:,1);
  V(:,i)=q(:,1);
  c1=d(1,1)^2;
   for k=1:ky
	ay=cumuly(k)+1:cumuly(k+1);
	ny=V(ay,i)'*V(ay,i);
	cri(k,i)=ny*c1;
	v(ay,i)=V(ay,i)/sqrt(ny);
     Y(:,ay)=Y(:,ay)-Y(:,ay)*v(ay,i)*v(ay,i)';
   end
end	
cov2=cri./n^2;	

endfunction
