///////////////////////////////////////////////////////////////////////////////
// concorCANO
//                                                                             
//       Canonical analysis between one subset X and several subsets Yj
//  
//                                                                             
//                                                 SUCCESSIVE SOLUTIONS 
//       
// Relative proximity analysis of subsets to one set
///////////////////////////////////////////////////////////////////////////////

function [P,u,Py,ry,v,V,rho2]=concorCANO(X,Y,py,r)

///////////////////////////////////////////////////////////////////////////////
// X and Y are 2 data matrices, n x p and n x q, of p variables and q variables
// (centered).
// The row vector py contains the ky numbers of variables of the ky subsets of Y.
// sum(py)=q. Y is the concatenated matrix of matrices Yj, j=1,...,ky.            
// r is the wanted number of solutions [0<r< 1+min(rank(Yj'X))].
// 
// P is a matrix with n rows which is obtained from X by standardization.
// Py is a matrix with n rows which contains the standardized matrices Pyj 
// respectively associated to Yj. The number of the columns of Pyj is ry(j).
// The r columns of P*u are the r canonical variables relative to X.
// v contains ky blocks vj, each of them with r columns. v1=v(1:ry(1),:),
// v2=v(ry(1)+1:ry(1)+ry(2),:), ...
// Pyj*vj are the r canonical variables of Yj, respectively associated with the
// r canonical variables of X.
//
// For each set, the canonical variables are standardized and 2 by 2 zero
// correlated.
// rho2 (ky x r) contains the canonical coefficients rho2(Yjvj(:,k),Pu(:,k)) 
// associated to the solution k, k=1, ...r.
// The total sum of squared correlations of the the first solution is maximal.
//
// Py*V contains (for each k) weighted means of the Pyj*vj 
// (the weights, are for each fixed k, rho(Yjvj(:,k),Pu(:,k)).
// For the set of r solutions, the matrix (P*u)'Py*V is diagonal.
// The matrices (Pu)'Pyjvj are triangular.
//
//
// diag((P*u)'*Py*V/n)'.^2 = sum(rho2,1)
//
// Hanafi & Lafosse (2001) in Revue de Statistique Appliquee vol.49, n.1.
// hanafi@enitiaa-nantes.fr
// Roger.Lafosse@lsp.ups-tlse.fr
/////////////////////////////////////////////////////////////////////////
[n,q]=size(Y);
 if (sum(py) ~= q)
    display(' py IS NOT SUITABLE')
    return
 else
 end

// rk here is the maximal value. rk may be excessive.
// Idem for the values of ry. 
rk=rank(X);
[pp,d,qq]=svd(X);
P=pp(:,1:rk)*sqrt(n);
ky=size(py,2);

ry=zeros(1,ky);Py=[];
cy=[0,cumsum(py)];
for j=1:ky
ry(j)=rank(Y(:,cy(j)+1:cy(j+1)));
[pp,d,qq]=svd(Y(:,cy(j)+1:cy(j+1)));
Py=[Py,pp(:,1:ry(j))*sqrt(n)];
end
q=sum(ry);
PPy=Py;

rho2=zeros(ky,r);
cumuly=cumsum([0,ry]);
u=zeros(rk,r);
V=zeros(q,r);
v=V;

for i=1:r
  [pp,d,qq]=svd(P'*PPy);
  u(:,i)=pp(:,1);
  V(:,i)=qq(:,1);
  for k=1:ky
	ay=cumuly(k)+1:cumuly(k+1);
	ny=V(ay,i)'*V(ay,i);
	rho2(k,i)=ny*(d(1,1)/n)^2;
	v(ay,i)=V(ay,i)/sqrt(ny);
     PPy(:,ay)=PPy(:,ay)-PPy(:,ay)*v(ay,i)*v(ay,i)';
  end
end	


endfunction
