//////////////////////////////////////////////////////////////////////////////
// concorPLS
//                                                                              
//                      Regression PLS of subsets Yj
//  
//                           by one set X.                                       
//                                                                              
//                                                 SUCCESSIVE SOLUTIONS 
//       
// When concorREG is not suitable for reasons usual in PLS
//////////////////////////////////////////////////////////////////////////////

function [Cx,u,v,V,cri]=concorPLS(X,Y,py,r)

///////////////////////////////////////////////////////////////////////////////
// X and Y are 2 data matrices, n x p and n x q, of p variables and q variables
// (centered).
// The row vector py contains the ky numbers qj of variables of the ky subsets
// of Y. sum(py)=q. Y is the concatenated matrix of matrices Yj, j=1,...,ky.
//              
// r is the wanted number of solutions [0<r< 1+min(rank(Yj'X))], and thus  
// rmax < 1+min(min(py),n).
//  
// The columns of Cx (n x r) are the r explanatory components.
// They are associated to :
// V (q x r) the orthonormed global axes of Y, and also to : 
// kx matrices vj (qj xr), blocks of v (q x r), normed partial axes of
// the respective Yj, j=1,...,ky.
// cri (ky x r) contains the explained variances :
// for the solution column k, ky values rho2(Yjvj(:,k),Cx(:,k)) var(Yjvj(:,k)).
//
// u (p x r) is not very useful : only u(,1) contains the scores of the variables
// of X (Cx(:,1) is the standardized component X*u(:,1) ).
//
// Each solution associates one explanatory component to ky explained components Yjvj.
// A column of Y*V is a mean component of partial explained components Yjvj.
// For a set of r solutions, the matrix Cx'*Y*V  is
// triangular : on average, the explanatory component of one solution
// is not linked with the mean explanation given by the following solutions.
//
// The definition of the explanatory components does not depend on the partition
// vector py.
// The orthogonality constraints for successive solutions are :
// V'*V = Ir and Cx'*Cx =Ir.
//
// diag(Cx'*y*V/n)'.^2 = sum(cri,1) = sumj diag(Cx'*Yj*vj/n)'.^2 
//
// Another PLS in Vivien & Sabatier (2001) Revue de Stat appli. vol.49 n.1
// myrtille.vivien@univ-montp1.fr
////////////////////////////////////////////////////////////////////////////////////
[n,q]=size(Y);
 if (sum(py) ~= q)
    display(' py IS NOT SUITABLE')
    return
 else
 end

ky=size(py,2);
cri=zeros(ky,r);
cumuly=cumsum([0,py]);
u=zeros(size(X,2),r);
V=zeros(q,r);
v=V;
Cx=zeros(n,r);

for i=1:r
  [pp,d,qq]=svd(X'*Y);
  u(:,i)=pp(:,1);
  V(:,i)=qq(:,1);
  Cx(:,i)=X*u(:,i);
  nx=(Cx(:,i)'*Cx(:,i));Cx(:,i)=Cx(:,i)/sqrt(nx);
  X=X-Cx(:,i)*Cx(:,i)'*X;
  Cx(:,i)=Cx(:,i)*sqrt(n);
  Y=Y-Y*V(:,i)*V(:,i)';

  for k=1:ky
	ay=cumuly(k)+1:cumuly(k+1);
	ny=V(ay,i)'*V(ay,i);
	cri(k,i)=ny*d(1,1)^2/(n*nx);
	v(ay,i)=V(ay,i)/sqrt(ny);
  end

end	

endfunction
