/////////////////////////////////////////////////////////////////////////////
// concorREG                                                                             
//                      Regression of subsets Yj
//  
//                           by one set X.                                       
//                                                                              
//                                                 SUCCESSIVE SOLUTIONS        
// Redundancy analysis
/////////////////////////////////////////////////////////////////////////////

function [P,u,v,V,cri]=concorREG(X,Y,py,r)

///////////////////////////////////////////////////////////////////////////////
// X and Y are 2 data matrices, n x p and n x q, of p variables and q variables
// (centered).
// The row vector py contains the ky numbers of variables of the ky subsets of Y.
// sum(py)=q. Y is the concatenated matrix of matrices Yj, j=1,...,ky.            
// r is the wanted number of solutions [0<r< 1+min(rank(Yj'X))], and thus  
// rmax < 1+min(min(py),n). 
// 
// u (p x r) is an orthonormed system associated to :
// V (q x r) the orthonormed global axes of Y, and also to :
// each of the ky blocks vj of v (q x r), which contains r orthonormed partial
// axes relative to Yj, j=1,...,ky.
//
// cri (ky x r) contains the explained variances :
// rho2(Yjvj(:,k),Pu(:,k)) var(Yjvj(:,k)) for the solution k, k=1, ...r.
// sum(cri,2) contains r successive optimized criteria, with these
// orthogonality constraints between solutions : uj'*uj = Ir, j=1, ...kx.
// The total explained variance by the first solution is maximal.
//
// Each solution associates one explanatory component to ky explained components Yjvj.
// Y*V(:,k) is a mean component of partial explained components Yjvj(:,k).
// The columns of P*u are the explanatory components. The matrix X has been
// standardized for obtaining the matrix P.
// For a set of r solutions, the matrix (Pu)'YV is diagonal : on average, 
// the explanatory component of one solution is only linked with the components
// explained by this explanatory, and is not linked with the explained
// components of the other solutions.
// The matrices (Pu)'Yjvj are triangular : the explanatory component of one solution
// is not linked with each of the partial components explained in the following
// solutions.
//
// The definition of the explanatory components depends on the partition vector
// py from the second solution.
//
// diag((P*u)'*y*V/n)'.^2 = sum(cri,1)
//
// Hanafi & Lafosse (2001) in Revue de Statistique Appliquee vol.49, n.1.
// hanafi@enitiaa-nantes.fr
// Roger.Lafosse@lsp.ups-tlse.fr
////////////////////////////////////////////////////////////////////////////////
[n,q]=size(Y);
 if (sum(py) ~= q)
    display(' py IS NOT SUITABLE')
    return
 else
 end

// rk here is the maximal value. A smaller value may be better in some cases. 
rk=rank(X);
[pp,d,qq]=svd(X);
P=pp(:,1:rk)*sqrt(n);

ky=size(py,2);
cri=zeros(ky,r);
cumuly=cumsum([0,py]);
u=zeros(rk,r);
V=zeros(q,r);
v=V;

for i=1:r
  [pp,d,qq]=svd(P'*Y);
  u(:,i)=pp(:,1);
  V(:,i)=qq(:,1);
  for k=1:ky
	ay=cumuly(k)+1:cumuly(k+1);
	ny=V(ay,i)'*V(ay,i);
	cri(k,i)=ny*(d(1,1)/n)^2;
	v(ay,i)=V(ay,i)/sqrt(ny);
     Y(:,ay)=Y(:,ay)-Y(:,ay)*v(ay,i)*v(ay,i)';
  end
end	


endfunction

